# -*- coding: utf-8 -*-

"""Console script for pymyair."""

from __future__ import absolute_import

import click
from pymyair.pymyair import MyAir
import json
import requests
import pprint


@click.group(invoke_without_command=True)
@click.argument('ip')
@click.option('--port', default=2025, help='port of MyPlace service')
@click.option(
    '--aircon',
    default="ac1",
    help='aircon id, only required if you have multiple AC units')
@click.pass_context
def cli(ctx, ip, port, aircon):
    """Console script for pymyair."""
    ctx.obj = {}
    if ctx.invoked_subcommand is None:
        click.echo('Add --help for available commands')
    else:
        try:
            ctx.obj['myair'] = MyAir(host=ip, port=port, aircon=aircon)
            ctx.obj['myair'].update()
        except requests.exceptions.ConnectionError:
            raise click.BadParameter(
                "Connection failed, please check IP: %s" %
                (ip))


@cli.command()
@click.pass_context
def on(ctx):
    """turns on AC"""
    ctx.obj['myair'].mode = 'on'


@cli.command()
@click.pass_context
def off(ctx):
    """turns off AC"""
    ctx.obj['myair'].mode = 'off'


@cli.command()
@click.pass_context
def cool(ctx):
    """switches AC to cooling"""
    ctx.obj['myair'].mode = 'cool'


@cli.command()
@click.pass_context
def heat(ctx):
    """switches AC to heating"""
    ctx.obj['myair'].mode = 'heat'


@cli.command()
@click.pass_context
def vent(ctx):
    """switching AC to venting (if available)"""
    ctx.obj['myair'].mode = 'vent'


@cli.command()
@click.pass_context
def dry(ctx):
    """switching AC to drying (if available)"""
    ctx.obj['myair'].mode = 'dry'


@cli.command()
@click.pass_context
def mode(ctx):
    """returns current operating mode"""
    click.echo(ctx.obj['myair'].mode)


@cli.command()
@click.pass_context
def zones(ctx):
    """returns json of zone information"""
    click.echo(json.dumps(ctx.obj['myair'].zones, sort_keys=True, indent=2))


# @cli.command()
# @click.argument('speed')
# @click.pass_context
# def fan(ctx,speed):
#     """sets fan speed"""
#     ctx.obj['myair'].fanspeed = speed

@cli.command()
@click.option('--speed',
              type=click.Choice(['low',
                                 'medium',
                                 'high',
                                 'auto']),
              help='optional, [low|medium|high|auto]')
@click.pass_context
def fan(ctx, speed):
    '''if speed supplied, updates fan speed'''
    if speed:
        ctx.obj['myair'].fanspeed = speed
    else:
        click.echo(ctx.obj['myair'].fanspeed)


def validate_temp_value(ctx, param, value):
    pp = pprint.PrettyPrinter()
    pp.pprint(ctx.params)
    if value not in range(16, 32):  # and ctx.params['value'] in range(0,100):
        raise click.BadParameter(
            'supply settemp or value percentage for zone, not both')
    return value


def validate_zone_set(ctx, param, value):
    if value not in range(1, 10):
        raise click.BadParameter('zone required: [1-10]')
    return value


@cli.command()
@click.option(
    '--zone',
    type=click.IntRange(
        1,
        10),
    callback=validate_zone_set,
    help='zone id as integer [1-10]')
@click.option(
    '--state', type=click.Choice(['on', 'off']), help='optional, [on|off]')
@click.option(
    '--temp',
    type=click.IntRange(
        16,
        32),
    callback=validate_temp_value,
    help='temperature set point as integer in C [16-32]')
@click.option('--value', type=click.IntRange(0, 100),
              help='percentage open as integer [0-100]')
@click.pass_context
def set(ctx, zone, state, value, temp):
    """sets zone parameters

    --temp or --value is required depending on
    whether a temperature sensor is installed
    """
    ctx.obj['myair'].setZone(id=zone, state=state, set_temp=temp, value=value)


@cli.command()
@click.option('--zone', type=click.IntRange(1, 10),
              help='zone id as integer [1-10]')
@click.pass_context
def myzone(ctx, zone):
    '''if zone supplied, updates myZone'''
    if zone in range(1, 10):
        ctx.obj['myair'].myzone = zone
    else:
        click.echo(ctx.obj['myair'].myzone)


if __name__ == '__main__':
    cli(obj={})



# -*- coding: utf-8 -*-

"""Main module."""

# API based on Specification published at
# http://advantageair.proboards.com/thread/2/api-document
# Requires signup, this is intended so that you can be informed of changes
# to the spec
from __future__ import absolute_import

import requests
import json
import time

FAN_MAP = {'low': 1, 'medium': 2, 'high': 3, 'auto': 4}
MODE_MAP = {'cool': 1, 'heat': 2, 'fan': 3, 'dry': 5}

# TODO: check fan mode (docs say vent, is it the same?)


class MyAir(object):
    def __init__(self, host, port=2025, aircon="ac1"):
        '''Params are host, port(default=2025), aircon(default="ac1")'''
        self._host = host
        self._port = port
        self._aircon = aircon
        self._system = None
        self._zones = None

    def _request(self, request):

        url = "http://%s:%s/%s" % (self._host, self._port, request)

        r = requests.get(url)
        r.raise_for_status()

        data = r.content.decode('utf-8')

        try:
            entry = json.loads(data)

            return entry

        except json.AttributeError:
            raise Exception("Found malformed JSON at %s: %s", url, data)

    def update(self):
        '''Fetch current state from MyAir'''
        # getSystemData can be somewhat unreliable
        # so we retry a number of times before giving up
        req = self._request("getSystemData")
        retry = 0
        sleep = 0.01
        while len(req) == 0 and retry < 30:
            retry = retry + 1
            if sleep < 5:
                sleep = sleep * 2
            time.sleep(sleep)
            req = self._request("getSystemData")
        if len(req) == 0:
            raise Exception("getSystemData failed")

        self._system = req
        return self._system

    @property
    def aircons(self):
        '''Returns a list of aircon identifiers,
        useful if you have more than one AC unit
        attached to your MyAir system'''
        return json.dumps(self.system['aircons'].keys)

    @property
    def zones(self):
        if not self._system:
            raise Exception('Need to call update()')
        return self.system['aircons'][self._aircon]['zones']

    @zones.setter
    def zones(self, update):
        '''Set zone(s) state, setTemp/value (Temp vs Percentage) using structure
        {"z02":{"state":"open"}}
        {"z04":{"setTemp":25},"z06":{"setTemp":25}}
        '''
        setjson = {self._aircon: {"zones": update}}
        valid_json = json.loads(setjson)

        self._request("setAircon?json=%s" % json.dumps(valid_json))

    def setTemp(self, set_temp=None):
        '''Set global temperature'''

        if set_temp in range(16, 32):
            setjson = "{\"ac1\":{\"info\":{\"setTemp\":%d}}}" % set_temp
        else:
            raise Exception(
                "temp needs to be in range 16-32, temp: %s" % set_temp)

        self._request("setAircon?json=%s" % setjson)

    def setZone(self, id, state=None, set_temp=None, value=None):
        '''Set zone state[on|off], set_temp or value (temp sensor/percent)'''

        # TODO: update string concat to json object

        if state == 'open' or (
                state != 'close' and
                state != 'off' and
                state != '0' and
                state):
            state_value = 'open'
        else:
            state_value = 'close'

        zoneid = "z%02d" % id

        setjson = "{\"%s\":{\"zones\":{\"%s\":{" % (self._aircon, zoneid)

        if state is not None and state_value:
            setjson += "\"state\":\"%s\"" % state_value

        if self.system['aircons'][self._aircon]['zones'][zoneid]['type'] > 0:
            # temperature sensor available
            if value and set_temp is None:
                raise Exception(
                    "zone requires setting by set_temp as it has temp sensor")
            if set_temp in range(16, 32):
                if state:
                    setjson += ","
                setjson += "\"setTemp\":%d" % set_temp
            else:
                raise Exception(
                    "temp needs to be in range 16-32, temp: %s" %
                    set_temp)
        else:
            # percentage set value for zone
            if value:
                if state:
                    setjson += ","
                setjson += "\"value\":%d" % value

        setjson += "}}}}"
        self._request("setAircon?json=%s" % setjson)

    @property
    def system(self):
        '''Returns complete system information from MyPlace Service'''
        if not self._system:
            raise Exception('Need to call update()')
        return self._system

    # Accessors for global mode, fanspeed
    @property
    def mode(self):
        '''Current set mode, 'off', 'heat', 'cool', 'dry', 'vent'.'''
        mode = self.system['aircons'][self._aircon]['info']['mode']
        onoff = self.system['aircons'][self._aircon]['info']['state']
        if onoff == 'off':
            return 'off'
        elif onoff == 'on':
            return mode

    @mode.setter
    def mode(self, mode):
        '''Set mode of AC: 'off', 'heat', 'cool', 'dry', 'vent'.'''
        # Ensure we're online
        self.system

        if (mode in ('off', 'on')):
            setjson = {self._aircon: {"info": {"state": mode}}}
            self._system['aircons'][self._aircon]['info']['state'] = mode
        elif mode in MODE_MAP:
            setjson = {self._aircon: {"info": {"state": "on", "mode": mode}}}
            self._system['aircons'][self._aircon]['info']['mode'] = mode
            self._system['aircons'][self._aircon]['info']['state'] = 'on'
        else:
            raise Exception('invalid state: %s' % mode)

        return self._request("setAircon?json=%s" % json.dumps(setjson))

    @property
    def myzone(self):
        '''Current MyZone'''
        return self.system['aircons'][self._aircon]['info']['myZone']

    @myzone.setter
    def myzone(self, val):
        '''Set MyZone [1-10]'''
        self.system

        if val in range(1, 10):
            setjson = {self._aircon: {"info": {"myZone": val}}}
            self._system['aircons'][self._aircon]['info']['myZone'] = val
            return self._request("setAircon?json=%s" % json.dumps(setjson))
        else:
            raise Exception('invalid myZone')

    @property
    def fanspeed(self):
        '''Current fanspeed, 'low', 'medium', 'high', 'auto'.'''
        return self.system['aircons'][self._aircon]['info']['fan']

    @fanspeed.setter
    def fanspeed(self, val):
        '''Set fanspeed:  'low', 'medium', 'high', 'auto'.'''
        # Ensure we're online
        self.system

        if val in FAN_MAP:
            setjson = {self._aircon: {"info": {"fan": val}}}
            self._system['aircons'][self._aircon]['info']['fan'] = val
            return self._request("setAircon?json=%s" % json.dumps(setjson))
        else:
            raise Exception('invalid fanSpeed, valid speeds:' + str(FAN_MAP))
